@@ -4,10 +4,25 @@
 
 #![warn(rust_2018_idioms)]
 #![warn(missing_docs)]
+#![deny(rustdoc::broken_intra_doc_links)]
 
 //! *sqrid* provides square grid coordinates and related operations,
 //! in a single-file create, with no dependencies.
 //!
+//! It's easier to explain the features of this crate in terms of the
+//! types it provides:
+//! - [`Qa`]: position, as absolute coordinates in a grid of fixed
+//!   size. The dimensions of the grid are const generics type
+//!   parameters; invalid coordinates can't be created.
+//! - [`Qr`]: "movement", relative coordinates. These are the cardinal
+//!   (and intercardinal) directions.
+//!   Addition is implemented in the form of `Qa + Qr = Option<Qa>`,
+//!   which can be `None` if the result is outside the grid.
+//! - [`Grid`]: a `Qa`-indexed array.
+//!
+//! All these types have the standard `iter`, `iter_mut`, `extend`,
+//! `as_ref`, and conversion operations that should be expected.
+//!
 //! # `Qa`: absolute coordinates, position
 //!
 //! The [`Qa`] type represents an absolute position in a square
@@ -26,8 +41,8 @@
 //! - Using one of the const associated items:
 //!   ```rust
 //!   type Qa = sqrid::Qa<6, 7>;
-//!   const MyFirst : Qa = Qa::FIRST;
-//!   const MyLast : Qa = Qa::LAST;
+//!   const MY_FIRST : Qa = Qa::FIRST;
+//!   const MY_LAST : Qa = Qa::LAST;
 //!   ```
 //! - Using `try_from` with a `(i16, i16)` tuple or a tuple reference:
 //!   ```rust
@@ -35,10 +50,9 @@
 //!   use std::error::Error;
 //!
 //!   type Qa = sqrid::Qa<6, 7>;
-//!   const MyFirst : Qa = Qa::FIRST;
 //!
 //!   fn main() -> Result<(), Box<dyn Error>> {
-//!       let qa1 = Qa::try_from((2_i16, 3_i16))?;
+//!       let qa1 = Qa::try_from((2_u16, 3_u16))?;
 //!
 //!       println!("qa1: {}", qa1);
 //!       Ok(())
@@ -65,9 +79,8 @@
 //! # `Qr`: relative coordinates, direction, movement
 //!
 //! This type represents a relative movement of one square. It can
-//! only be one of the 4 cardinal directions (N, E, S, W) if the
-//! `DIAG` argument is false, or one of the 8 directions when it's
-//! true.
+//! only be one of the 8 cardinal and intercardinal directions (N, NE,
+//! E, SE, S, SW, W, NW).
 //!
 //! It's a building block for paths, iterating on a [`Qa`] neighbors,
 //! etc. It effectively represents the edges in a graph where the
@@ -78,8 +91,8 @@
 //!   cardinal directions (recommended):
 //!   ```rust
 //!   use sqrid::Qr;
-//!   const Right : Qr<false> = Qr::E;
-//!   const Down : Qr<false> = Qr::S;
+//!   const RIGHT : Qr = Qr::E;
+//!   const DOWN : Qr = Qr::S;
 //!   ```
 //! - Using `try_from` with a `(i16, i16)` tuple or a tuple reference:
 //!   ```rust
@@ -90,29 +103,65 @@
 //!
 //!   fn main() -> Result<(), Box<dyn Error>> {
 //!       // Re-create West:
-//!       let qr1 = Qr::<false>::try_from((0_i16, -1_i16))?;
+//!       let qr1 = Qr::try_from((0_i8, -1_i8))?;
 //!       // Re-create Northeast:
-//!       let qr2 = Qr::<true>::try_from((-1_i16, 1_i16))?;
+//!       let qr2 = Qr::try_from((-1_i8, 1_i8))?;
 //!       Ok(())
 //!   }
 //!   ```
-//! - Calling [`Qr::new`], which checks the bounds in const contexts:
-//!   ```rust
-//!   use sqrid::Qr;
-//!   const DOWNRIGHT : Qr<true> = Qr::<true>::new::<1, 1>();
-//!   ```
-//!   The following, for instance, doesn't compile due to the argument
-//!   being false, which means that diagonals are not supported:
-//!   ```compile_fail
-//!   use sqrid::Qr;
-//!   const DOWNRIGHT : Qr<false> = Qr::<false>::new::<1, 1>();
-//!   ```
 //! - Calling [`Qr::iter`] to iterate all directions:
 //!   ```rust
-//!   for qr in sqrid::Qr::<true>::iter() {
+//!   for qr in sqrid::Qr::iter::<true>() {
 //!       println!("{}", qr);
 //!   }
 //!   ```
+//!   The const argument to Qr::iter signals it to iterate over the
+//!   intercardinal directions too. Passing `false` gets us only the 4
+//!   cardinal directions.
+//!
+//! # `Grid`: a `Qa`-indexed array
+//!
+//! A grid is a generic array that can be indexed by a [`Qa`]
+//!
+//! We can create the type from a suitable [`Qa`] type by using the
+//! [`grid_create`] macro. We can then interact with specific lines
+//! with [`Grid::line`] and [`Grid::line_mut`], or with the whole
+//! underlying array with [`as_ref`](std::convert::AsRef) and
+//! [`as_mut`](std::convert::AsMut).
+//!
+//! Usage example:
+//!
+//! ```rust
+//! type Qa = sqrid::Qa<3, 3>;
+//! type Grid = sqrid::grid_create!(i32, Qa);
+//!
+//! // The grid create macro above is currently equivalent to:
+//! type Grid2 = sqrid::Grid<i32, { Qa::WIDTH }, { Qa::HEIGHT },
+//!                               { (Qa::WIDTH * Qa::HEIGHT) as usize }>;
+//!
+//! // We can create grids from iterators via `collect`:
+//! let mut gridnums = (0..9).collect::<Grid>();
+//!
+//! // Iterate on their members:
+//! for i in &gridnums {
+//!     println!("i {}", i);
+//! }
+//!
+//! // Change the members in a loop:
+//! for i in &mut gridnums {
+//!     *i *= 10;
+//! }
+//!
+//! // Iterate on (coordinate, member) tuples:
+//! for (qa, &i) in gridnums.iter_qa() {
+//!     println!("[{}] = {}", qa, i);
+//! }
+//!
+//! // And we can always use `as_ref` or `as_mut` to interact with the
+//! // inner array directly. To reverse it, for example, with the
+//! // [`std::slice::reverse`] function:
+//! gridnums.as_mut().reverse();
+//! ```
 
 pub mod _sqrid;
 pub use self::_sqrid::*;
